/*
 *  Copyright 2010 Alexey Frishman (alexey.frishman@gmail.com)
 *  
 *  Licensed under the Apache License, Version 2.0 (the "License"); you may not
 *  use this file except in compliance with the License. You may obtain a copy of
 *  the License at
 *  
 *  http://www.apache.org/licenses/LICENSE-2.0
 *  
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *  License for the specific language governing permissions and limitations under
 *  the License.
 * 
 */

package org.json4g.serialization;

import java.util.List;

import com.google.gwt.json.client.JSONValue;
import com.gwtent.reflection.client.Field;

/**
 * Consists of a number of concrete serializers. It just delegates serialization/deserialization to the first serializer, which
 * supports the format of the given value and then skips all next serializers. Thus the order of concrete serializers is important.
 */
public class JsonCompositeSerializer implements JsonSerializer {

    private List<JsonSerializer> serializers;

    public JsonCompositeSerializer(List<JsonSerializer> serializers) {
        this.serializers = serializers;
    }

    public JSONValue serialize(Object object) {
        JSONValue result = null;
        for (JsonSerializer serializer : serializers) {
            if (serializer.supports(object)) {
                result = serializer.serialize(object);
                break;
            }
        }

        return result;
    }

    @Override
    public Object deserialize(JSONValue json, Field field) {
        Object result = null;
        for (JsonSerializer serializer : serializers) {
            if (serializer.supports(json, field)) {
                result = serializer.deserialize(json, field);
                break;
            }
        }
        return result;
    }

    @Override
    public boolean supports(Object object) {
        boolean result = false;
        for (JsonSerializer s : serializers) {
            if (s.supports(object)) {
                result = true;
                break;
            }
        }
        return result;
    }

    @Override
    public boolean supports(JSONValue json, Field field) {
        boolean result = false;
        for (JsonSerializer s : serializers) {
            if (s.supports(json, field)) {
                result = true;
                break;
            }
        }
        return result;
    }

}
